@@ -3,7 +3,7 @@
 #include <meteoland.h>
 using namespace Rcpp;
 
-
+const double SIGMA_Wm2 = 5.67*1e-8;
 /***
  * FUNCTIONS FOR LIGHT EXTINCTION
  */
@@ -322,8 +322,7 @@ NumericVector layerSunlitFraction(NumericMatrix LAIme, NumericMatrix LAImd, Nume
 // [[Rcpp::export("light_instantaneousLightExtinctionAbsortion")]]
 List instantaneousLightExtinctionAbsortion(NumericMatrix LAIme, NumericMatrix LAImd, NumericMatrix LAImx, 
                                            NumericVector kPAR, NumericVector alphaSWR, NumericVector gammaSWR,
-                                           DataFrame ddd, NumericVector LWR_diffuse, 
-                                           int ntimesteps = 24, String canopyMode= "sunshade", double trunkExtinctionFraction = 0.1) {
+                                           DataFrame ddd, int ntimesteps = 24, double trunkExtinctionFraction = 0.1) {
 
   int numCohorts = LAIme.ncol();
   int nz = LAIme.nrow();
@@ -346,7 +345,7 @@ List instantaneousLightExtinctionAbsortion(NumericMatrix LAIme, NumericMatrix LA
     gammaPAR[c] = gammaSWR[c]*0.8; // (PAR albedo 80% of SWR albedo)
     kbvec[c] = kb;
     alphaLWR[c] = 0.97; //Longwave coefficients
-    kLWR[c] = 0.8;
+    // kLWR[c] = 0.8;
   }
   
   //Average radiation extinction fractions for direct and diffuse PAR/SWR radiation and LWR radiation
@@ -355,25 +354,31 @@ List instantaneousLightExtinctionAbsortion(NumericMatrix LAIme, NumericMatrix LA
   NumericVector Idfpar = layerIrradianceFraction(LAIme,LAImd,LAImx, kPAR, alphaPAR, trunkExtinctionFraction);
   NumericVector Ibfswr = layerIrradianceFraction(LAIme,LAImd,LAImx, kbvec, alphaSWR,trunkExtinctionFraction);
   NumericVector Idfswr = layerIrradianceFraction(LAIme,LAImd,LAImx, kSWR, alphaSWR, trunkExtinctionFraction);
-  NumericVector Idflwr_td = layerIrradianceFraction(LAIme,LAImd,LAImx, kLWR, alphaLWR, trunkExtinctionFraction);
-  NumericVector Idflwr_bu = layerIrradianceFractionBottomUp(LAIme,LAImd,LAImx, kLWR, alphaLWR, trunkExtinctionFraction);
+  // NumericVector Idflwr_td = layerIrradianceFraction(LAIme,LAImd,LAImx, kLWR, alphaLWR, trunkExtinctionFraction);
+  // NumericVector Idflwr_bu = layerIrradianceFractionBottomUp(LAIme,LAImd,LAImx, kLWR, alphaLWR, trunkExtinctionFraction);
   
   //Fraction of incoming diffuse/direct SWR radiation and LWR radiation reaching the ground
   double gbf = groundDirectIrradianceFraction(LAIme,LAImd,LAImx, kbvec, alphaSWR, trunkExtinctionFraction);
   double gdf = groundDiffuseIrradianceFraction(LAIme,LAImd,LAImx, kSWR, trunkExtinctionFraction);
-  double glwr = groundDiffuseIrradianceFraction(LAIme,LAImd,LAImx, kLWR, trunkExtinctionFraction);
-  
+  // double glwr = groundDiffuseIrradianceFraction(LAIme,LAImd,LAImx, kLWR, trunkExtinctionFraction);
+  // 
   //Average sunlit fraction
   NumericVector fsunlit = layerSunlitFraction(LAIme, LAImd, kbvec);
 
   // Rcout<<rad<<" "<< solarElevation[0]<<" "<<SWR_direct[0]<<"\n";
 
-  List abs_PAR_SL_list(ntimesteps);
-  List abs_SWR_SL_list(ntimesteps);
-  List abs_PAR_SH_list(ntimesteps);
-  List abs_SWR_SH_list(ntimesteps);
-  List abs_LWR_SL_list(ntimesteps);
-  List abs_LWR_SH_list(ntimesteps);
+  List abs_PAR_SL_COH_list(ntimesteps);
+  List abs_SWR_SL_COH_list(ntimesteps);
+  List abs_PAR_SH_COH_list(ntimesteps);
+  List abs_SWR_SH_COH_list(ntimesteps);
+  // List abs_LWR_SL_COH_list(ntimesteps);
+  // List abs_LWR_SH_COH_list(ntimesteps);
+  List abs_PAR_SL_ML_list(ntimesteps);
+  List abs_SWR_SL_ML_list(ntimesteps);
+  List abs_PAR_SH_ML_list(ntimesteps);
+  List abs_SWR_SH_ML_list(ntimesteps);
+  // List abs_LWR_SL_ML_list(ntimesteps);
+  // List abs_LWR_SH_ML_list(ntimesteps);
   NumericVector abs_SWR_can(ntimesteps,0.0), abs_LWR_can(ntimesteps,0.0);
   NumericVector abs_SWR_soil(ntimesteps,0.0), abs_LWR_soil(ntimesteps,0.0);
   for(int n=0;n<ntimesteps;n++) {
@@ -388,86 +393,142 @@ List instantaneousLightExtinctionAbsortion(NumericMatrix LAIme, NumericMatrix LA
                                                       Ibfswr, Idfswr, solarElevation[n],
                                                       LAIme, LAImd, 
                                                       kbvec,  kSWR, alphaSWR, gammaSWR);
-    //Calculate sky LWR absorved radiation 
-    NumericMatrix abs_LWR_sky = cohortDiffuseAbsorbedRadiation(LWR_diffuse[n], 
-                                                      Idflwr_td,
-                                                      LAIme, LAImd, 
-                                                      kLWR, alphaLWR, gammaLWR);
-    //Calculate ground-originated LWR absorbed (assumes same input as for sky but from below)
-    NumericMatrix abs_LWR_ground = cohortDiffuseAbsorbedRadiation(LWR_diffuse[n], 
-                                                               Idflwr_bu,
-                                                               LAIme, LAImd, 
-                                                               kLWR, alphaLWR, gammaLWR);
     NumericMatrix mswrsl = abs_SWR["I_sunlit"];
     NumericMatrix mswrsh = abs_SWR["I_shade"];
     NumericMatrix mparsl = abs_PAR["I_sunlit"];
     NumericMatrix mparsh = abs_PAR["I_shade"];
 
     // Rcout << SWR_direct[n]*1000.0 << " "<< SWR_diffuse[n]*1000.0 << " " << LWR_diffuse[n] << "\n";
-    if(canopyMode=="multilayer") {
-      abs_PAR_SL_list[n] = mparsl;
-      abs_PAR_SH_list[n] = mparsh;
-      abs_SWR_SL_list[n] = mswrsl;
-      abs_SWR_SH_list[n] = mswrsh;
-      NumericMatrix vlwr_sl(nz, numCohorts), vlwr_sh(nz, numCohorts);
-      for(int c=0;c<numCohorts;c++){
-        for(int i=0;i<nz;i++){
-          vlwr_sl(i,c)+= (abs_LWR_sky(i,c)+abs_LWR_ground(i,c))*LAIme(i,c)*fsunlit[i]; //Add top-down and bottom-up lwr 
-          vlwr_sh(i,c)+= (abs_LWR_sky(i,c)+abs_LWR_ground(i,c))*LAIme(i,c)*(1.0 - fsunlit[i]); 
-        }
+    //Multiple layer
+    abs_PAR_SL_ML_list[n] = mparsl;
+    abs_PAR_SH_ML_list[n] = mparsh;
+    abs_SWR_SL_ML_list[n] = mswrsl;
+    abs_SWR_SH_ML_list[n] = mswrsh;
+    //Aggregate light (PAR, SWR, LWR) for sunlit leaves and shade leaves
+    NumericVector vparsl(numCohorts,0.0), vparsh(numCohorts,0.0);
+    NumericVector vswrsl(numCohorts,0.0), vswrsh(numCohorts,0.0);
+    for(int c=0;c<numCohorts;c++){
+      for(int i=0;i<nz;i++){
+        mparsl(i,c)=mparsl(i,c)*LAIme(i,c)*fsunlit[i];
+        mparsh(i,c)=mparsh(i,c)*LAIme(i,c)*(1.0-fsunlit[i]);
+        mswrsl(i,c)=mswrsl(i,c)*LAIme(i,c)*fsunlit[i];
+        mswrsh(i,c)=mswrsh(i,c)*LAIme(i,c)*(1.0-fsunlit[i]);
+        vparsl[c]+=mparsl(i,c);
+        vparsh[c]+=mparsh(i,c);
+        vswrsl[c]+=mswrsl(i,c);
+        vswrsh[c]+=mswrsh(i,c);
       }
-      abs_LWR_SL_list[n] = vlwr_sl;
-      abs_LWR_SH_list[n] = vlwr_sh;
-    }  else if (canopyMode=="sunshade"){
-      //Aggregate light (PAR, SWR, LWR) for sunlit leaves and shade leaves
-      NumericVector vparsl(numCohorts,0.0), vparsh(numCohorts,0.0);
-      NumericVector vswrsl(numCohorts,0.0), vswrsh(numCohorts,0.0);
-      NumericVector vlwr_sl(numCohorts,0.0), vlwr_sh(numCohorts,0.0);
-      for(int c=0;c<numCohorts;c++){
-        for(int i=0;i<nz;i++){
-          vparsl[c]+=mparsl(i,c)*LAIme(i,c)*fsunlit[i];
-          vparsh[c]+=mparsh(i,c)*LAIme(i,c)*(1.0-fsunlit[i]);
-          vswrsl[c]+=mswrsl(i,c)*LAIme(i,c)*fsunlit[i];
-          vswrsh[c]+=mswrsh(i,c)*LAIme(i,c)*(1.0-fsunlit[i]);
-          vlwr_sl[c]+= (abs_LWR_sky(i,c)+abs_LWR_ground(i,c))*LAIme(i,c)*fsunlit[i]; //Add top-down and bottom-up lwr 
-          vlwr_sh[c]+= (abs_LWR_sky(i,c)+abs_LWR_ground(i,c))*LAIme(i,c)*(1.0 - fsunlit[i]); 
-        }
-        // Rcout<<"Hola "<<vswrsl[c]<<" "<<vswrsh[c]<<" "<<vparsl[c]<<" "<<vparsh[c]<<"\n";
-      }
-      
-      abs_PAR_SL_list[n] = vparsl;
-      abs_PAR_SH_list[n] = vparsh;
-      abs_SWR_SL_list[n] = vswrsl;
-      abs_SWR_SH_list[n] = vswrsh;
-      abs_LWR_SL_list[n] = vlwr_sl;
-      abs_LWR_SH_list[n] = vlwr_sh;
+      // Rcout<<"Hola "<<vswrsl[c]<<" "<<vswrsh[c]<<" "<<vparsl[c]<<" "<<vparsh[c]<<"\n";
     }
+    
+    abs_PAR_SL_COH_list[n] = vparsl;
+    abs_PAR_SH_COH_list[n] = vparsh;
+    abs_SWR_SL_COH_list[n] = vswrsl;
+    abs_SWR_SH_COH_list[n] = vswrsh;
     //Calculate canopy absorbed radiation (includes absortion by trunks in winter)
     double abs_dir_swr = SWR_direct[n]*1000.0*(1.0 - gbf); //W/m2
     double abs_dif_swr = SWR_diffuse[n]*1000.0*(1.0 - gdf); //W/m2
     abs_SWR_can[n] = abs_dir_swr+abs_dif_swr;
-    abs_LWR_can[n] = LWR_diffuse[n]*(1.0-glwr);
     // Rcout<<n<<" "<< abs_SWR_can[n]<< " "<<abs_LWR_can[n]<<"\n";
     //Calculate soil absorved radiation
     abs_SWR_soil[n] = 0.90*((gbf*SWR_direct[n]*1000.0)+(gdf*SWR_diffuse[n]*1000.0)); //10% reflectance for SWR (Geiger, The climate near the ground)
-    abs_LWR_soil[n] = 0.95*(LWR_diffuse[n]*glwr); //5% soil reflectance for LWR
-    
+
     // Rcout<<n<<" PAR : "<<(PAR_direct[n]*1000.0)+(PAR_diffuse[n]*1000.0)<<" SWR: "<<(SWR_direct[n]*1000.0)+(SWR_diffuse[n]*1000.0) <<" can: "<< abs_SWR_can[n]<< " soil: "<< abs_SWR_soil[n]<<" LWR: "<< (LWR_diffuse[n]) <<" can: "<< abs_LWR_can[n]<< " soil: "<< abs_LWR_soil[n]<<"\n";
   }
+  List multilayer = List::create(_["PAR_SL"] = abs_PAR_SL_ML_list,
+                                 _["PAR_SH"] = abs_PAR_SH_ML_list,
+                                 _["SWR_SL"] = abs_SWR_SL_ML_list,
+                                 _["SWR_SH"] = abs_SWR_SH_ML_list);
+  List sunshade = List::create(_["PAR_SL"] = abs_PAR_SL_COH_list,
+                               _["PAR_SH"] = abs_PAR_SH_COH_list,
+                               _["SWR_SL"] = abs_SWR_SL_COH_list,
+                               _["SWR_SH"] = abs_SWR_SH_COH_list);
   List res = List::create(_["kb"] = kb,
                           _["fsunlit"] = fsunlit,
-                          _["PAR_SL"] = abs_PAR_SL_list,
-                          _["PAR_SH"] = abs_PAR_SH_list,
-                          _["SWR_SL"] = abs_SWR_SL_list,
-                          _["SWR_SH"] = abs_SWR_SH_list,
-                          _["LWR_SL"] = abs_LWR_SL_list,
-                          _["LWR_SH"] = abs_LWR_SH_list,
+                          _["multilayer"] = multilayer,
+                          _["sunshade"] = sunshade,
                           _["SWR_can"] = abs_SWR_can,
-                          _["LWR_can"] = abs_LWR_can, //Includes only absorbed LWR from sky
                           _["SWR_soil"] = abs_SWR_soil,
-                          _["LWR_soil"] = abs_LWR_soil, //Includes only absorbed LWR from sky
                           _["gbf"] = gbf, //ground direct SWR fraction
-                          _["gdf"] = gdf, //ground diffuse SWR fraction
-                          _["glwr"] = glwr); //ground diffuse LWR fraction
+                          _["gdf"] = gdf); //ground diffuse LWR fraction
   return(res);
 }
+
+/**
+ *  LWR model of Ma and Liu (2019), based on Flerchinger et al (2009)
+ *  
+ *  Ma Y, Liu H (2019) An Advanced Multiple-Layer Canopy Model in the WRF Model With Large-Eddy Simulations to Simulate Canopy Flows and Scalar Transport Under Different Stability Conditions. J Adv Model Earth Syst 11:2330–2351. https://doi.org/10.1029/2018MS001347
+ *  Flerchinger GN, Xiao W, Sauer TJ, Yu Q (2009) Simulation of within-canopy radiation exchange. NJAS - Wageningen J Life Sci 57:5–15. https://doi.org/10.1016/j.njas.2009.07.004
+ */
+// [[Rcpp::export("light_longwaveRadiationSHAW")]]
+List longwaveRadiationSHAW(NumericMatrix LAIme, NumericMatrix LAImd, NumericMatrix LAImx, 
+                           double LWRatm, double Tsoil, NumericVector Tair, double trunkExtinctionFraction = 0.1) {
+  int ncoh = LAIme.ncol();
+  int ncanlayers = Tair.size();
+  NumericVector Lup(ncanlayers), Ldown(ncanlayers), Lnet(ncanlayers);
+  NumericVector tau(ncanlayers), sumTauComp(ncanlayers);
+  NumericMatrix lai_ij(ncanlayers, ncoh);
+  NumericMatrix tauM(ncanlayers, ncoh);
+  NumericMatrix LnetM(ncanlayers, ncoh);
+  if(ncoh>0) LnetM.attr("dimnames") = List::create(seq(1,ncanlayers), seq(1,ncoh));
+  
+  double Kdlw = 0.7815; //Extinction coefficient fo LWR
+  double eps_c = 0.97;
+  double eps_g = 0.97;
+  //Transmissivity
+  for(int i=0;i<ncanlayers;i++) {
+    double lai_layer = 0.0;
+    sumTauComp[i] = 0.0;
+    for(int j=0;j<ncoh;j++) {
+      lai_ij(i,j) = std::max(LAIme(i,j)+LAImd(i,j), trunkExtinctionFraction*LAImx(i,j));
+      tauM(i,j) = exp(-Kdlw*lai_ij(i,j));
+      sumTauComp[i] += (1.0-tauM(i,j)); 
+      lai_layer +=lai_ij(i,j);
+    }
+    tau[i] = exp(-Kdlw*lai_layer);
+  }
+  //Downwards
+  for(int i=(ncanlayers-1);i>=0;i--) {
+    double Ldown_upper = 0.0;
+    if(i==(ncanlayers-1)) Ldown_upper = LWRatm;
+    else Ldown_upper = Ldown[i+1];
+    Ldown[i] = tau[i]*Ldown_upper + (1.0 - tau[i])*eps_c*SIGMA_Wm2*pow(Tair[i]+273.16,4.0);
+  }
+  //Upwards
+  double Lup_g = (1.0 - eps_g)*Ldown[0] + eps_g*SIGMA_Wm2*pow(Tsoil+273.16,4.0);
+  for(int i=0;i<ncanlayers;i++) {
+    double Lup_lower = 0.0;
+    if(i==0) Lup_lower = Lup_g;
+    else Lup_lower = Lup[i-1];
+    Lup[i] = tau[i]*Lup_lower + (1.0 - tau[i])*eps_c*SIGMA_Wm2*pow(Tair[i]+273.16,4.0);
+  }
+  //Net
+  for(int i=0;i<ncanlayers;i++) {
+    double Lup_lower = 0.0;
+    if(i==0) Lup_lower = Lup_g;
+    else Lup_lower = Lup[i-1];
+    Lnet[i] = eps_c*(1.0 - tau[i])*(Ldown[i]+Lup_lower - 2.0*SIGMA_Wm2*pow(Tair[i]+273.16,4.0));
+    for(int j=0;j<ncoh;j++) {
+      LnetM(i,j) =0.0;
+      if(LAIme(i,j)>0.0) {
+        LnetM(i,j) =  Lnet[i]*((1.0-tauM(i,j))/sumTauComp[i]);
+        //Correct for the fact that extinction included all leaves and energy balance is on expanded leaves
+        LnetM(i,j) = LnetM(i,j)*(LAIme(i,j)/lai_ij(i,j)); 
+        
+      } 
+    }
+  }
+  double Lnet_g = eps_g*(Ldown[0] - SIGMA_Wm2*pow(Tsoil+273.16,4.0));
+  double Lnet_c = sum(Lnet);
+  DataFrame LWR = DataFrame::create(_["Ldown"] = Ldown, 
+                                    _["Lup"] = Lup,
+                                    _["Lnet"] = Lnet);
+  return(List::create(_["LWR_layer"] = LWR,
+                      _["Ldown_ground"] = Ldown[0],
+                      _["Lup_ground"] = Lup_g,
+                      _["Lnet_ground"] = Lnet_g,
+                      _["Ldown_canopy"] = LWRatm,
+                      _["Lup_canopy"] = Lup[(ncanlayers-1)],
+                      _["Lnet_canopy"] = Lnet_c,
+                      _["Lnet_cohort_layer"] = LnetM));
+}
